@@ -14,52 +14,45 @@ import datetime
 import itertools
 import mock
 
-
 from oslo_config import cfg
-from wsme import types as wtypes
+from oslo_serialization import jsonutils
 
-from watcher.api.controllers.v1 import action_plan as api_action_plan
 from watcher.applier import rpcapi as aapi
 from watcher.common import utils
 from watcher.db import api as db_api
 from watcher import objects
 from watcher.tests.api import base as api_base
-from watcher.tests.api import utils as api_utils
-from watcher.tests import base
 from watcher.tests.objects import utils as obj_utils
 
 
-class TestActionPlanObject(base.TestCase):
-
-    def test_action_plan_init(self):
-        act_plan_dict = api_utils.action_plan_post_data()
-        del act_plan_dict['state']
-        del act_plan_dict['audit_id']
-        del act_plan_dict['first_action_id']
-        act_plan = api_action_plan.ActionPlan(**act_plan_dict)
-        self.assertEqual(wtypes.Unset, act_plan.state)
-
-
 class TestListActionPlan(api_base.FunctionalTest):
 
+    def setUp(self):
+        super(TestListActionPlan, self).setUp()
+        obj_utils.create_test_goal(self.context)
+        obj_utils.create_test_strategy(self.context)
+        obj_utils.create_test_audit(self.context)
+
     def test_empty(self):
         response = self.get_json('/action_plans')
         self.assertEqual([], response['action_plans'])
 
     def _assert_action_plans_fields(self, action_plan):
-        action_plan_fields = ['state']
+        action_plan_fields = [
+            'uuid', 'audit_uuid', 'strategy_uuid', 'strategy_name',
+            'state', 'global_efficacy', 'efficacy_indicators']
         for field in action_plan_fields:
             self.assertIn(field, action_plan)
 
     def test_one(self):
-        action_plan = obj_utils.create_action_plan_without_audit(self.context)
+        action_plan = obj_utils.create_test_action_plan(self.context)
         response = self.get_json('/action_plans')
         self.assertEqual(action_plan.uuid,
                          response['action_plans'][0]["uuid"])
         self._assert_action_plans_fields(response['action_plans'][0])
 
     def test_one_soft_deleted(self):
-        action_plan = obj_utils.create_action_plan_without_audit(self.context)
+        action_plan = obj_utils.create_test_action_plan(self.context)
         action_plan.soft_delete()
         response = self.get_json('/action_plans',
                                  headers={'X-Show-Deleted': 'True'})
@@ -71,10 +64,18 @@ class TestListActionPlan(api_base.FunctionalTest):
         self.assertEqual([], response['action_plans'])
 
     def test_get_one_ok(self):
-        action_plan = obj_utils.create_action_plan_without_audit(self.context)
+        action_plan = obj_utils.create_test_action_plan(self.context)
+        obj_utils.create_test_efficacy_indicator(
+            self.context, action_plan_id=action_plan['id'])
         response = self.get_json('/action_plans/%s' % action_plan['uuid'])
         self.assertEqual(action_plan.uuid, response['uuid'])
         self._assert_action_plans_fields(response)
+        self.assertEqual(
+            [{'description': 'Test indicator',
+              'name': 'test_indicator',
+              'value': 0.0,
+              'unit': '%'}],
+            response['efficacy_indicators'])
 
     def test_get_one_with_first_action(self):
         action_plan = obj_utils.create_test_action_plan(self.context)
@@ -85,7 +86,7 @@ class TestListActionPlan(api_base.FunctionalTest):
         self._assert_action_plans_fields(response)
 
     def test_get_one_soft_deleted(self):
-        action_plan = obj_utils.create_action_plan_without_audit(self.context)
+        action_plan = obj_utils.create_test_action_plan(self.context)
         action_plan.soft_delete()
         response = self.get_json('/action_plans/%s' % action_plan['uuid'],
                                  headers={'X-Show-Deleted': 'True'})
@@ -97,15 +98,14 @@ class TestListActionPlan(api_base.FunctionalTest):
         self.assertEqual(404, response.status_int)
 
     def test_detail(self):
-        action_plan = obj_utils.create_test_action_plan(self.context,
-                                                        audit_id=None)
+        action_plan = obj_utils.create_test_action_plan(self.context)
         response = self.get_json('/action_plans/detail')
         self.assertEqual(action_plan.uuid,
                          response['action_plans'][0]["uuid"])
         self._assert_action_plans_fields(response['action_plans'][0])
 
     def test_detail_soft_deleted(self):
-        action_plan = obj_utils.create_action_plan_without_audit(self.context)
+        action_plan = obj_utils.create_test_action_plan(self.context)
         action_plan.soft_delete()
         response = self.get_json('/action_plans/detail',
                                  headers={'X-Show-Deleted': 'True'})
@@ -126,7 +126,7 @@ class TestListActionPlan(api_base.FunctionalTest):
     def test_many(self):
         action_plan_list = []
         for id_ in range(5):
-            action_plan = obj_utils.create_action_plan_without_audit(
+            action_plan = obj_utils.create_test_action_plan(
                 self.context, id=id_, uuid=utils.generate_uuid())
             action_plan_list.append(action_plan.uuid)
         response = self.get_json('/action_plans')
@@ -137,10 +137,10 @@ class TestListActionPlan(api_base.FunctionalTest):
     def test_many_with_soft_deleted_audit_uuid(self):
         action_plan_list = []
         audit1 = obj_utils.create_test_audit(self.context,
-                                             id=1,
+                                             id=2,
                                              uuid=utils.generate_uuid())
         audit2 = obj_utils.create_test_audit(self.context,
-                                             id=2,
+                                             id=3,
                                              uuid=utils.generate_uuid())
 
         for id_ in range(0, 2):
@@ -163,7 +163,7 @@ class TestListActionPlan(api_base.FunctionalTest):
 
         for id_ in range(0, 2):
             action_plan = response['action_plans'][id_]
-            self.assertEqual(None, action_plan['audit_uuid'])
+            self.assertIsNone(action_plan['audit_uuid'])
 
         for id_ in range(2, 4):
             action_plan = response['action_plans'][id_]
@@ -172,8 +172,9 @@ class TestListActionPlan(api_base.FunctionalTest):
     def test_many_with_audit_uuid(self):
         action_plan_list = []
         audit = obj_utils.create_test_audit(self.context,
+                                            id=2,
                                             uuid=utils.generate_uuid())
-        for id_ in range(5):
+        for id_ in range(2, 5):
             action_plan = obj_utils.create_test_action_plan(
                 self.context, id=id_, uuid=utils.generate_uuid(),
                 audit_id=audit.id)
@@ -186,14 +187,16 @@ class TestListActionPlan(api_base.FunctionalTest):
     def test_many_with_audit_uuid_filter(self):
         action_plan_list1 = []
         audit1 = obj_utils.create_test_audit(self.context,
+                                             id=2,
                                              uuid=utils.generate_uuid())
-        for id_ in range(5):
+        for id_ in range(2, 5):
             action_plan = obj_utils.create_test_action_plan(
                 self.context, id=id_, uuid=utils.generate_uuid(),
                 audit_id=audit1.id)
             action_plan_list1.append(action_plan.uuid)
 
         audit2 = obj_utils.create_test_audit(self.context,
+                                             id=3,
                                              uuid=utils.generate_uuid())
         action_plan_list2 = []
         for id_ in [5, 6, 7]:
@@ -210,7 +213,7 @@ class TestListActionPlan(api_base.FunctionalTest):
     def test_many_without_soft_deleted(self):
         action_plan_list = []
         for id_ in [1, 2, 3]:
-            action_plan = obj_utils.create_action_plan_without_audit(
+            action_plan = obj_utils.create_test_action_plan(
                 self.context, id=id_, uuid=utils.generate_uuid())
             action_plan_list.append(action_plan.uuid)
         for id_ in [4, 5]:
@@ -225,11 +228,11 @@ class TestListActionPlan(api_base.FunctionalTest):
     def test_many_with_soft_deleted(self):
         action_plan_list = []
         for id_ in [1, 2, 3]:
-            action_plan = obj_utils.create_action_plan_without_audit(
+            action_plan = obj_utils.create_test_action_plan(
                 self.context, id=id_, uuid=utils.generate_uuid())
             action_plan_list.append(action_plan.uuid)
         for id_ in [4, 5]:
-            action_plan = obj_utils.create_action_plan_without_audit(
+            action_plan = obj_utils.create_test_action_plan(
                 self.context, id=id_, uuid=utils.generate_uuid())
             action_plan.soft_delete()
             action_plan_list.append(action_plan.uuid)
@@ -241,8 +244,9 @@ class TestListActionPlan(api_base.FunctionalTest):
 
     def test_many_with_sort_key_audit_uuid(self):
         audit_list = []
-        for id_ in range(5):
+        for id_ in range(2, 5):
             audit = obj_utils.create_test_audit(self.context,
+                                                id=id_,
                                                 uuid=utils.generate_uuid())
             obj_utils.create_test_action_plan(
                 self.context, id=id_, uuid=utils.generate_uuid(),
@@ -251,14 +255,13 @@ class TestListActionPlan(api_base.FunctionalTest):
 
         response = self.get_json('/action_plans/?sort_key=audit_uuid')
 
-        self.assertEqual(5, len(response['action_plans']))
+        self.assertEqual(3, len(response['action_plans']))
         uuids = [s['audit_uuid'] for s in response['action_plans']]
         self.assertEqual(sorted(audit_list), uuids)
 
     def test_links(self):
         uuid = utils.generate_uuid()
-        obj_utils.create_action_plan_without_audit(self.context,
-                                                   id=1, uuid=uuid)
+        obj_utils.create_test_action_plan(self.context, id=1, uuid=uuid)
         response = self.get_json('/action_plans/%s' % uuid)
         self.assertIn('links', response.keys())
         self.assertEqual(2, len(response['links']))
@@ -269,7 +272,7 @@ class TestListActionPlan(api_base.FunctionalTest):
 
     def test_collection_links(self):
         for id_ in range(5):
-            obj_utils.create_action_plan_without_audit(
+            obj_utils.create_test_action_plan(
                 self.context, id=id_, uuid=utils.generate_uuid())
         response = self.get_json('/action_plans/?limit=3')
         self.assertEqual(3, len(response['action_plans']))
@@ -281,9 +284,8 @@ class TestListActionPlan(api_base.FunctionalTest):
         cfg.CONF.set_override('max_limit', 3, 'api',
                               enforce_type=True)
         for id_ in range(5):
-            obj_utils.create_action_plan_without_audit(
-                self.context, id=id_, uuid=utils.generate_uuid(),
-                audit_id=None)
+            obj_utils.create_test_action_plan(
+                self.context, id=id_, uuid=utils.generate_uuid())
         response = self.get_json('/action_plans')
         self.assertEqual(3, len(response['action_plans']))
 
@@ -295,7 +297,10 @@ class TestDelete(api_base.FunctionalTest):
 
     def setUp(self):
         super(TestDelete, self).setUp()
-        self.action_plan = obj_utils.create_action_plan_without_audit(
+        obj_utils.create_test_goal(self.context)
+        obj_utils.create_test_strategy(self.context)
+        obj_utils.create_test_audit(self.context)
+        self.action_plan = obj_utils.create_test_action_plan(
             self.context)
         p = mock.patch.object(db_api.BaseConnection, 'destroy_action_plan')
         self.mock_action_plan_delete = p.start()
@@ -351,7 +356,10 @@ class TestPatch(api_base.FunctionalTest):
 
     def setUp(self):
         super(TestPatch, self).setUp()
-        self.action_plan = obj_utils.create_action_plan_without_audit(
+        obj_utils.create_test_goal(self.context)
+        obj_utils.create_test_strategy(self.context)
+        obj_utils.create_test_audit(self.context)
+        self.action_plan = obj_utils.create_test_action_plan(
             self.context, state=objects.action_plan.State.RECOMMENDED)
         p = mock.patch.object(db_api.BaseConnection, 'update_action_plan')
         self.mock_action_plan_update = p.start()
@@ -368,7 +376,7 @@ class TestPatch(api_base.FunctionalTest):
         test_time = datetime.datetime(2000, 1, 1, 0, 0)
         mock_utcnow.return_value = test_time
 
-        new_state = 'DELETED'
+        new_state = objects.action_plan.State.DELETED
         response = self.get_json(
             '/action_plans/%s' % self.action_plan.uuid)
         self.assertNotEqual(new_state, response['state'])
@@ -444,7 +452,7 @@ class TestPatch(api_base.FunctionalTest):
         response = self.patch_json(
             '/action_plans/%s' % self.action_plan.uuid,
             [{'path': '/state', 'value': new_state,
-             'op': 'replace'}])
+              'op': 'replace'}])
         self.assertEqual('application/json', response.content_type)
         self.assertEqual(200, response.status_code)
         applier_mock.assert_called_once_with(mock.ANY,
@@ -490,11 +498,17 @@ class TestPatchStateTransitionDenied(api_base.FunctionalTest):
              "new_state": new_state} not in ALLOWED_TRANSITIONS
     ]
 
+    def setUp(self):
+        super(TestPatchStateTransitionDenied, self).setUp()
+        obj_utils.create_test_goal(self.context)
+        obj_utils.create_test_strategy(self.context)
+        obj_utils.create_test_audit(self.context)
+
     @mock.patch.object(
         db_api.BaseConnection, 'update_action_plan',
         mock.Mock(side_effect=lambda ap: ap.save() or ap))
     def test_replace_state_pending_denied(self):
-        action_plan = obj_utils.create_action_plan_without_audit(
+        action_plan = obj_utils.create_test_action_plan(
             self.context, state=self.original_state)
 
         initial_ap = self.get_json('/action_plans/%s' % action_plan.uuid)
@@ -512,24 +526,6 @@ class TestPatchStateTransitionDenied(api_base.FunctionalTest):
         self.assertTrue(response.json['error_message'])
 
 
-class TestPatchStateDeletedNotFound(api_base.FunctionalTest):
-
-    @mock.patch.object(
-        db_api.BaseConnection, 'update_action_plan',
-        mock.Mock(side_effect=lambda ap: ap.save() or ap))
-    def test_replace_state_pending_not_found(self):
-        action_plan = obj_utils.create_action_plan_without_audit(
-            self.context, state=objects.action_plan.State.DELETED)
-
-        response = self.get_json(
-            '/action_plans/%s' % action_plan.uuid,
-            expect_errors=True
-        )
-        self.assertEqual(404, response.status_code)
-        self.assertEqual('application/json', response.content_type)
-        self.assertTrue(response.json['error_message'])
-
-
 class TestPatchStateTransitionOk(api_base.FunctionalTest):
 
     scenarios = [
@@ -541,23 +537,98 @@ class TestPatchStateTransitionOk(api_base.FunctionalTest):
         for transition in ALLOWED_TRANSITIONS
     ]
 
+    def setUp(self):
+        super(TestPatchStateTransitionOk, self).setUp()
+        obj_utils.create_test_goal(self.context)
+        obj_utils.create_test_strategy(self.context)
+        obj_utils.create_test_audit(self.context)
+
     @mock.patch.object(
         db_api.BaseConnection, 'update_action_plan',
         mock.Mock(side_effect=lambda ap: ap.save() or ap))
     @mock.patch.object(aapi.ApplierAPI, 'launch_action_plan', mock.Mock())
     def test_replace_state_pending_ok(self):
-        action_plan = obj_utils.create_action_plan_without_audit(
+        action_plan = obj_utils.create_test_action_plan(
             self.context, state=self.original_state)
 
         initial_ap = self.get_json('/action_plans/%s' % action_plan.uuid)
 
         response = self.patch_json(
             '/action_plans/%s' % action_plan.uuid,
-            [{'path': '/state', 'value': self.new_state,
-             'op': 'replace'}])
+            [{'path': '/state', 'value': self.new_state, 'op': 'replace'}])
         updated_ap = self.get_json('/action_plans/%s' % action_plan.uuid)
 
         self.assertNotEqual(self.new_state, initial_ap['state'])
         self.assertEqual(self.new_state, updated_ap['state'])
         self.assertEqual('application/json', response.content_type)
         self.assertEqual(200, response.status_code)
+
+
+class TestActionPlanPolicyEnforcement(api_base.FunctionalTest):
+
+    def setUp(self):
+        super(TestActionPlanPolicyEnforcement, self).setUp()
+        obj_utils.create_test_goal(self.context)
+        obj_utils.create_test_strategy(self.context)
+        obj_utils.create_test_audit(self.context)
+
+    def _common_policy_check(self, rule, func, *arg, **kwarg):
+        self.policy.set_rules({
+            "admin_api": "(role:admin or role:administrator)",
+            "default": "rule:admin_api",
+            rule: "rule:defaut"})
+        response = func(*arg, **kwarg)
+        self.assertEqual(403, response.status_int)
+        self.assertEqual('application/json', response.content_type)
+        self.assertTrue(
+            "Policy doesn't allow %s to be performed." % rule,
+            jsonutils.loads(response.json['error_message'])['faultstring'])
+
+    def test_policy_disallow_get_all(self):
+        self._common_policy_check(
+            "action_plan:get_all", self.get_json, '/action_plans',
+            expect_errors=True)
+
+    def test_policy_disallow_get_one(self):
+        action_plan = obj_utils.create_test_action_plan(self.context)
+        self._common_policy_check(
+            "action_plan:get", self.get_json,
+            '/action_plans/%s' % action_plan.uuid,
+            expect_errors=True)
+
+    def test_policy_disallow_detail(self):
+        self._common_policy_check(
+            "action_plan:detail", self.get_json,
+            '/action_plans/detail',
+            expect_errors=True)
+
+    def test_policy_disallow_update(self):
+        action_plan = obj_utils.create_test_action_plan(self.context)
+        self._common_policy_check(
+            "action_plan:update", self.patch_json,
+            '/action_plans/%s' % action_plan.uuid,
+            [{'path': '/state',
+              'value': objects.action_plan.State.DELETED,
+              'op': 'replace'}],
+            expect_errors=True)
+
+    def test_policy_disallow_delete(self):
+        action_plan = obj_utils.create_test_action_plan(self.context)
+        self._common_policy_check(
+            "action_plan:delete", self.delete,
+            '/action_plans/%s' % action_plan.uuid, expect_errors=True)
+
+
+class TestActionPlanPolicyEnforcementWithAdminContext(TestListActionPlan,
+                                                      api_base.AdminRoleTest):
+
+    def setUp(self):
+        super(TestActionPlanPolicyEnforcementWithAdminContext, self).setUp()
+        self.policy.set_rules({
+            "admin_api": "(role:admin or role:administrator)",
+            "default": "rule:admin_api",
+            "action_plan:delete": "rule:default",
+            "action_plan:detail": "rule:default",
+            "action_plan:get": "rule:default",
+            "action_plan:get_all": "rule:default",
+            "action_plan:update": "rule:default"})
